import { Session } from 'next-auth';
import crypto from 'crypto';
import { ApiKeyEntry } from './db';
import * as random_words from "random-words";
import { TEMP_EMAIL_DOMAIN } from './constants';
import { aliasAvailable } from './actions';

export function sha256sum(input: any) {
  const hash = crypto.createHash('sha256');
  hash.update(input);
  return hash.digest('hex');
}

export function isAdmin(session: Session | string | null) {
  let email = typeof session == 'string' ? session : session?.user?.email;
  return email && ["lea@amogus.cloud", "lexi@futacockinside.me"].includes(email); // todo
}

export function aliasesNeedApproval(session: Session | null) {
  return !isAdmin(session); // also todo
}

export function anonymizeApiKey(key: ApiKeyEntry): ApiKeyEntry {
  return {
    ...key,
    token: key.token.substring(0, 6) + "********",
  };
}

export async function generateAliasEmail(label: string, style: 'words' | 'random', labelAtEnd: boolean): Promise<string> {
  let email: string;
  do {
    let randomString: string;
    switch (style) {
      case 'words':
        randomString = random_words.generate(2).join('');
        break;
      case 'random':
        randomString = crypto
          .randomBytes(8)
          .toString('base64')
          .replace(/\W/, ''); // Delete special characters
        break;
      default:
        throw new Error("Invalid style");
    }
    email = `${labelAtEnd ? `${randomString}-${label}` : `${label}-${randomString}`}@${TEMP_EMAIL_DOMAIN}`;
  } while (!await aliasAvailable(email, true));

  return email;
}
